#!/usr/bin/env python

import argparse
from django.template.defaultfilters import slugify
from articles.models import Article
from publish.utils import parse_meta_and_article, filter_field
from publish.utils import ConfigurationError
from publish.conf import FIELD_TO_KWARG, SAVE_NEEDED, REQUIRED_FIELDS, DB
from publish.conf import FIELD_DEFAULTS


def setfield(article, name, value, debug=False):
    """
    Simple function to keep things DRY
    """
    key = FIELD_TO_KWARG.get(name, name)
    try:
        setattr(article, key, value)
        if debug:
            print 'Set `%s` (as `%s`) -> %s' % (name, key, value)
    except ValueError:
        raise ValueError('Set primary key on Article to modify `%s`' % key)

def publish(f, draft, by=None, publish=None, is_active=True, login_required=False, debug=False):
    """
    Publishes an article.

    :param f: The file to parse
    :type f: file
    :param by: Author username
    :type by: str
    :param draft: Save as draft?
    :type draft: bool
    :param draft: Article active?
    :type draft: bool
    :param draft: Require login?
    :type draft: bool
    :param publish: When to publish
    :type publish: datetime fmt=YYYY-MM-DD HH:MM
    :param debug: Print debug data
    :type debug: bool

    :returns: Saved :class:`Article`
    """
    meta, content = parse_meta_and_article(f.read())
    meta.update({
        'is_active': is_active,
        'login_required': login_required,
    })
    if draft:
        meta['status'] = 'Draft'
    if publish:
        meta['publish'] = publish
    if by:
        meta['by'] = by
    slug = slugify(meta['title'])
    # New or updated?
    articles = Article.objects.using(DB).filter(slug=slug)
    if len(articles) > 1:
        if not publish:
            raise ConfigurationError('Title ambiguous; supply publish date')
        articles.filter(publish_date=meta['publish'])
        article = articles[0]
    elif len(articles) == 1:
        article = articles[0]
    else:
        article = Article()

    article.content = content

    todo = []
    keys = list(meta.keys())
    for key in keys + [k for k in REQUIRED_FIELDS+FIELD_DEFAULTS.keys() if k not in keys]:
        value = filter_field(key, meta)
        if key not in SAVE_NEEDED:
            setfield(article, key, value, debug)
        else:
            todo.append((key, value))
    article.save(using=DB)

    for key, value in todo:
        setfield(article, key, value, debug)
    article.save(using=DB)

    return article

def main():
    """
    ``publish`` executable.
    """
    parser = argparse.ArgumentParser(description='Article Publisher')
    parser.add_argument('path', type=argparse.FileType('r'),
                        help='Relative or absolute path to article')
    parser.add_argument('--by -b', type=str, metavar='name',
                        help='Author (by name)', default='', dest='by')
    parser.add_argument('--draft -d', action='store_true', default=False,
                        help='Publish as a draft only', dest='draft')
    parser.add_argument('--noactive', action='store_false', default=True,
                        help='Make the article "inactive"', dest='is_active')
    parser.add_argument('--login', action='store_false', default=True,
                        help='Require a login to view the article',
                        dest='login_required')
    parser.add_argument('--publish -p', type=str, metavar='YYYY-MM-DD HH:MM',
                        help='When to publish the article (overrides in-file '
                        'value, if any)', default='', dest='publish')
    parser.add_argument('--debug', action='store_true', default=False,
                        help='Print debugging info', dest='debug')
    args = parser.parse_args()
    article = publish(args.path, args.draft, args.by, args.publish, args.is_active, args.login_required, args.debug)
    print '%s (pk=%d) saved as %s' % (article.title, article.pk, article.status.name)




from docutils import io
from docutils.core import publish_doctree, Publisher
from docutils.readers.doctree import Reader
from docutils.writers import html4css1
from publish.conf import FIELD_DEFAULTS, FILTER_DEFAULTS
from django.conf import settings
# also bootstraps pygment's docutils support
from articles.models import Article


class ConfigurationError(Exception): pass


class Writer(html4css1.Writer):
    def __init__(self):
        html4css1.Writer.__init__(self)
        self.translator_class = HTMLTranslator


class HTMLTranslator(html4css1.HTMLTranslator):
    def visit_section(self, node):
        self.section_level += 1

    def depart_section(self, node):
        self.section_level -= 1


def parts_from_doctree(document, destination_path=None,
                         writer=Writer(), writer_name='html4css2',
                         settings=None, settings_spec=None,
                         settings_overrides=None, config_section=None,
                         enable_exit_status=None):
    """
    A version of :func:`docutils.core.publish_from_doctree` that supplies
    the "parts" rather than a regular string representation.
    """
    reader = Reader(parser_name='null')
    pub = Publisher(reader, None, writer,
                    source=io.DocTreeInput(document),
                    destination_class=io.StringOutput, settings=settings)
    if not writer and writer_name:
        pub.set_writer(writer_name)
    pub.process_programmatic_settings(
        settings_spec, settings_overrides, config_section)
    pub.set_destination(None, destination_path)
    output = pub.publish()
    return pub.writer.parts

def parse_meta_and_article(content):
    """
    Parse out the meta declarations (tags, keywords, etc...) from the article
    and return both.

    :param content: Contents of the article file
    :type content: str
    :returns: dict(meta) and str(article)
    """
    meta_field_names = FILTER_DEFAULTS.keys()
    meta = {}
    article = publish_doctree(content)
    for where, node in enumerate(article.traverse()):
        if node.tagname == 'field':
            label = str(node.children[0].children[0])
            if label in meta_field_names:
                value = str(node.children[1].children[0].children[0])
                meta[label] = value
                node.parent.remove(node)
        elif node.tagname == 'title' and 'title' not in meta:
            value = str(node.children[0])
            meta['title'] = value
            node.parent.remove(node)
    
    raw = parts_from_doctree(article)
    return (meta, raw['fragment'])

def filter_field(key, fields, func=None):
    """
    Run a field through the given filter, or default if none is provided. If
    field doesn't exist, it will attempt to return the default value for that
    field.

    :param key: the field name to use
    :param fields: a dictionary of available fields in the form ``name: value``
    :returns: Filtered value
    :raises: KeyError in the event the field can't be found among the provided
        ones or the defaults
    """
    try:
        val = fields[key]
    except KeyError:
        default_fields = getattr(settings, 'PUBLISH_FIELD_DEFAULTS', {})
        FIELD_DEFAULTS.update(default_fields)
        try:
            val = FIELD_DEFAULTS[key]
        except KeyError:
            raise ConfigurationError('You must supply the `%s` field in the'
                            ' article, via settings.PUBLISH_FIELD_DEFAULTS, or'
                            ' via command line (where applicable)' % key)


    default_filters = getattr(settings, 'PUBLISH_FILTER_DEFAULTS', {})
    FILTER_DEFAULTS.update(default_filters)
    f = func or FILTER_DEFAULTS[key] 
    return f(val)




def article_slugs(text):
    """
    Retrieve a list of :class:`Article` objects based on field text
    (a return-delimeted list of slugs).

    :param text: The text, such as:
            
            my-best-post
            my-second-best-post

    :returns: ``QuerySet`` of :class:`Articles`
    """
    from publish.conf import DB
    from articles.models import Article

    slugs = text.split('\n')
    return Article.objects.using(DB).filter(slug__in=slugs)

def article_tags(tagstr):
    """
    Convert a space-delimited list of tags to their :class:`Tag` equivalients.

    TODO:   Improve tags to support multiple words (make ``django-articles`` use
            ``django-taggit``!)

    :param tagstr: Tags in the form, ``one two three``.
    """
    from publish.conf import DB
    from articles.models import Tag

    tag = lambda n: Tag.objects.using(DB).get_or_create(name=Tag.clean_tag(n))[0]
    return [tag(t) for t in tagstr.split()]




from datetime import datetime
from articles.models import ArticleStatus
from publish.filters import article_slugs, article_tags
from django.conf import settings
from django.contrib.auth.models import User


# Database to use
DB = getattr(settings, 'PUBLISH_DB', 'default')

# Some sane defaults
FIELD_DEFAULTS = {
    'use_addthis': False,
    'status': 'Finished',
    'addthis': 'nobody'
}

# Default filters / transformations for fields
FILTER_DEFAULTS = {
    'title': lambda title: title,
    'tags': article_tags,
    'keywords': lambda kw: kw,
    'description': lambda desc: desc.replace('\n', ' '),
    'follow-up': article_slugs,
    'related': article_slugs,
    'status': lambda status: ArticleStatus.objects.using(DB).get(name=status),
    'use_addthis': lambda a: bool(a),
    'addthis': lambda a: bool(a),
    'publish': lambda d: datetime.strptime(d, '%Y-%m-%d %H:%M'),
    'expire': lambda d: datetime.strptime(d, '%Y-%m-%d %H:%M'),
    'by': lambda a: a and User.objects.using(DB).get(username__iexact=a) or \
        User.objects.using(DB).get(username=getattr(
            settings, 'PUBLISH_FIELD_DEFAULTS', {}).get('by', 0)),
    'is_active': lambda a: a,
    'login_required': lambda a: a
}

# Many-to-Many fields require a saved object before they work
SAVE_NEEDED = ('follow-up', 'related', 'tags')

# `django-articles` needs these, at least
REQUIRED_FIELDS = ['title', 'by']

# Convert our tag names to field names
FIELD_TO_KWARG = {
    'use_addthis': 'use_addthis_button',
    'addthis': 'addthis_username',
    'by': 'author',
    'expire': 'expiration_date',
    'publish': 'publish_date',
    'follow-up': 'followup_for',
    'related': 'related_articles',
}




from distribute_setup import use_setuptools
use_setuptools()
import sys, os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS')).read()

version = '1.0'

install_requires = [
    'docutils==0.7',
    'argparse',
]

setup(name='django-publish',
    version=version,
    description="cli publishing of blog articles to a django-articles blog.",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.6',
        'Topic :: Text Processing :: Markup'
    ],
    keywords='django django-articles reST publish blog',
    author='Tom Davis',
    author_email='tom@recursivedream.com',
    url='http://recursivedream.com',
    license='BSD',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={
        'console_scripts':
            ['publish=publish:main']
    }
)




import os
from StringIO import StringIO
from django.test import TestCase
from django.conf import settings
from django.contrib.auth.models import User
from docutils.core import publish_doctree
from publish import publish
from publish.utils import parts_from_doctree, parse_meta_and_article, filter_field
from publish.utils import ConfigurationError
from articles.models import Article, ArticleStatus, Tag


# This shouldn't be necessary but I can't figure out why normal fixture loading
# doesn't seem to work for this particular app.
from django.conf import settings
here = os.path.abspath(os.path.split(__file__)[0])
setattr(settings, 'FIXTURE_DIRS', [os.path.join(here, '../fixtures')])


class PublishTests(TestCase):
    fixtures = ['test_articles.json']

    def setUp(self):
        self.raw_sample = open(os.path.join(here, 'sample.rst'), 'r').read()
        self.article = Article.objects.get(pk=1)
        self.meta, self.html = parse_meta_and_article(self.raw_sample)

    def test_parts_from_doctree(self):
        """
        Test parsing parts from a doctree
        """
        article = publish_doctree(self.raw_sample)
        parts = parts_from_doctree(article)
        self.assertTrue('fragment' in parts)
        frag = parts['fragment']
        self.assertTrue('Some content' in frag)
        self.assertTrue('Post description...' not in frag)

    def test_simple_meta(self):
        """
        Test simple meta-data conversions
        """
        keywords = filter_field('keywords', self.meta)
        description = filter_field('description', self.meta)
        title = filter_field('title', self.meta)
        self.assertEqual(keywords, self.article.keywords)
        self.assertEqual(description, self.article.description)
        self.assertEqual(title, self.article.title)

    def test_article_slugs(self):
        """
        Test ``article_slugs`` filter
        """
        self.assertEqual(filter_field('related', self.meta)[0], self.article)

    def test_status_default(self):
        """
        Test ``status`` filter and use of conf defaults
        """
        status = ArticleStatus.objects.get(name='Finished')
        self.assertEqual(filter_field('status', self.meta), status)

    def test_author(self):
        """
        Test ``author`` filter which results in a ConfigurationError by default
        """
        self.assertRaises(ConfigurationError, filter_field, 'author', self.meta)

    def test_tags(self):
        """
        Test ``tags`` filter
        """
        tagqs = Tag.objects.all()
        self.assertEqual(list(tagqs), filter_field('tags', self.meta))

    def test_publish_update(self):
        """
        Publish an article update
        """
        f = open(os.path.join(here, 'sample.rst'))

        publish(f, draft=True, login_required=True)
        updated_article = Article.objects.get(pk=1)
        self.assertFalse(self.article.login_required)
        self.assertTrue(updated_article.login_required)

    def test_publish_new(self):
        """
        Publish a new article
        """
        # Make a different title
        title = 'Here is another article'
        pubstr = '2020-09-01 12:00'
        pubdate = filter_field('publish', { 'publish': pubstr })
        contents = open(os.path.join(here, 'sample.rst')).read().replace(
            'This is my title', title)
        f = StringIO(contents)

        a = publish(f, draft=True, publish=pubstr, debug=True)
        self.assertNotEqual(a.pk, self.article.pk)
        self.assertEqual(a.title, title)
        self.assertEqual(a.publish_date, pubdate)
        self.assertTrue('Some content' in a.content)
        followups = a.followup_for.all()
        self.assertTrue(self.article.pk in [a.pk for a in followups], followups)
        related = a.related_articles.all()
        # We have to use titles due to ambiguous slug
        self.assertTrue(self.article.title in [b.title for b in related], related)

